@ECHO OFF
ECHO.
ECHO EncryptAll v4.1 (C) Created by CookieRevised

REM Logviewer exit codes
REM 0: file is zero length
REM 1: no parameters, /?, action successful
REM 4: error in parameters, password too short
REM 5: file is not a PLE
REM 8: wrong password
REM 9: input past end, file damaged?

REM ****************************************************************************
REM ****************************************************************************
REM ***                                                                      ***
REM ***  Syntax: EncryptAll [password]                                       ***
REM ***                                                                      ***
REM ***          If you whish you can enter the password as a parameter.     ***
REM ***          eg: EncryptAll My LoNg PaSSw0rD                             ***
REM ***          If you don't do this the password will be asked.            ***
REM ***                                                                      ***
REM ***                                                                      ***
REM ***  Description                                                         ***
REM ***  -----------                                                         ***
REM ***                                                                      ***
REM ***  This batch file will encrypt all the non-encrypted log files from   ***
REM ***  Messenger Plus! 4 or 5 which exist in the current directory and     ***
REM ***  in its subdirectories.                                              ***
REM ***                                                                      ***
REM ***  The non-encrypted file will be deleted after a log has succesfully  ***
REM ***  been encrypted, so only the encrypted file will remain.             ***
REM ***                                                                      ***
REM ***  Run this batch file from inside your log directory.                 ***
REM ***                                                                      ***
REM ***  --                                                                  ***
REM ***                                                                      ***
REM ***  The latest version can be downloaded from:                          ***
REM ***  http://www.msghelp.net/showthread.php?pid=725219#pid725219          ***
REM ***                                                                      ***
REM ***  Please do not alter anything in this file.                          ***
REM ***                                                                      ***
REM ***  I've put a lot of effort and hours of testing into making this      ***
REM ***  batch file for what seemed to be a simple task. Using a batch file  ***
REM ***  to perform this task isn't the best option, but it was still done   ***
REM ***  to show what can be achieved with advanced batch programming.       ***
REM ***                                                                      ***
REM ***  Each line and each method being used is done for a reason as there  ***
REM ***  were a lot of ceavats to solve. One would think you could do a lot  ***
REM ***  of things differently or shorter, but unfortunatly this is not the  ***
REM ***  case. As such, this batch file uses a lot of well known and less    ***
REM ***  known tricks and workarounds to do certain things.                  ***
REM ***                                                                      ***
REM ***  If you have any questions about why I did something in a certain    ***
REM ***  way or you have a better way, don't hesitate to contact me.         ***
REM ***                                                                      ***
REM ****************************************************************************
REM ****************************************************************************


REM ****************************************************************************
REM *** Check if the command extensions are enabled                          ***
REM ****************************************************************************

:: Very important checks which are too often forgotten.
:: We disable delayed expansion because that has serious problems when you use FOR loops with filenames.
:: (eg: files with "!" in their pathname will produce errors when used in loops with delayed expansion)
VERIFY EXTTEST 2>NUL
SETLOCAL ENABLEEXTENSIONS DISABLEDELAYEDEXPANSION
IF ERRORLEVEL 1 GOTO NOEXTENSIONS


REM ****************************************************************************
REM *** Check if the needed command programs are available                   ***
REM ****************************************************************************

FOR %%I IN (REG.EXE) DO SET CMDREG="%%~$PATH:I"
IF NOT EXIST %CMDREG% GOTO NOCMDPROGS

FOR %%I IN (ATTRIB.EXE) DO SET CMDATTRIB="%%~$PATH:I"
IF NOT EXIST %CMDATTRIB% GOTO NOCMDPROGS


REM ****************************************************************************
REM *** Check if we can create the needed temporary files                    ***
REM ****************************************************************************

:: Using ECHO.>FILE will produce an error when you don't have admin rights to alter the file on Vista/7, which can't be piped.
:: Using DEL and IF EXIST wont detect admin rights errors when the file isn't there to begin with and when you are later going to use ECHO.>FILE.
:: Using COPY has the advantage that you can pipe the error messages and that you can check an errorlevel which can't be done with DEL either.
FOR %%I IN (~EDAF.TMP ~EDAR.TMP ~EDAS.TMP) DO (
  %CMDATTRIB% %%I -H -S -R 1>NUL
  COPY NUL %%I 1>NUL 2>&1
  IF ERRORLEVEL 1 GOTO NOSAVING
  DEL %%I 1>NUL 2>&1
)


REM ****************************************************************************
REM *** Check if there are any non-encrypted logs                            ***
REM ****************************************************************************

:: Less known trick to do an IF THEN ELSE check on one command line.
:: Notice the use of || (which is the OR operator) which is executed when the first command returns an errorlevel greater than 0.
:: Use && (AND operator) to execute when the first command returns an errorlevel of 0.
:: Notice we excluded hidden files because the FOR loop, used later on, will not be able to detect those.
DIR *.HTML /S/A-D-H 1>NUL 2>&1 || DIR *.TXT /S/A-D-H 1>NUL 2>&1 || GOTO NOLOGS


REM ****************************************************************************
REM *** Get the path and filename of the log viewer for Plus! 4 and 5        ***
REM ****************************************************************************

:: Check if we are running the command line in native 64-bit mode. If so, redirect the registry key.
IF /I '%PROCESSOR_ARCHITECTURE%'=='AMD64' SET WOW64=WOW6432NODE\
IF /I '%PROCESSOR_ARCHITECTURE%'=='IA64' SET WOW64=WOW6432NODE\
:: The next check shouldn't be needed since Windows should transparently redirect registry keys when running in 32-bit mode on a 64-bit machine.
:: However, I'm unable to verify this for myself, so I leave this line in case it is needed.
:: When needed, simply remove the REM keyword at the beginning.
REM IF DEFINED PROCESSOR_ARCHITEW6432 SET WOW64=WOW6432NODE\

:: Trick to put reg.exe output into an environment variable, without using external batch files.
FOR /F "DELIMS=" %%I IN ('%%CMDREG%% QUERY "HKLM\SOFTWARE\%WOW64%Patchou\Messenger Plus! Live" /v AppDir 2^>NUL') DO SET LOGVIEWER4=%%I
SET LOGVIEWER4=%LOGVIEWER4:	=    %
SET LOGVIEWER4="%LOGVIEWER4:~24%\Log Viewer.exe"
IF NOT EXIST %LOGVIEWER4% SET LOGVIEWER4=

FOR /F "DELIMS=" %%I IN ('%%CMDREG%% QUERY "HKLM\SOFTWARE\%WOW64%Yuna Software\Messenger Plus!" /v AppDir 2^>NUL') DO SET LOGVIEWER5=%%I
SET LOGVIEWER5=%LOGVIEWER5:	=    %
SET LOGVIEWER5="%LOGVIEWER5:~24%\Log Viewer.exe"
IF NOT EXIST %LOGVIEWER5% SET LOGVIEWER5=


REM ****************************************************************************
REM *** Determine what version of Messenger Plus! to use                     ***
REM ****************************************************************************

:: Here we use some GOTO statements instead of a nested IF THEN ELSE because delayed expansion is disabled and otherwise the variables will not update properly.
:: Also notice the use of single quotes instead of double quotes. This is important because the path variable will already contain double quotes and otherwise
:: the command interpreter will not parse the command line properly.
IF NOT '%LOGVIEWER4%'=='' IF NOT '%LOGVIEWER5%'=='' GOTO MAKECHOICE
SET LOGVIEWER=%LOGVIEWER4%%LOGVIEWER5%
IF '%LOGVIEWER%'=='' GOTO NOPLUS
GOTO CONTINUE

:MAKECHOICE
>&3 ECHO.
>&3 ECHO Both Messenger Plus! 4 and Messenger Plus! 5 seem to be installed.
>&3 SET /P CHOICE=What version do you want to use to encrypt (4 or 5)? 
IF '%CHOICE%'=='4' SET LOGVIEWER=%LOGVIEWER4%
IF '%CHOICE%'=='5' SET LOGVIEWER=%LOGVIEWER5%
IF '%LOGVIEWER%'=='' GOTO NOINPUT
:CONTINUE


REM ****************************************************************************
REM *** Check if a password has been given, if not ask for one               ***
REM ****************************************************************************

SET PASSWORD=%*
IF '%PASSWORD%'=='' (
  :: Force output to the screen, even if output was redirected by the user.
  >&3 ECHO.
  >&3 SET /P PASSWORD=Enter the encryption password: 
)
IF '%PASSWORD%'=='' GOTO NOPASSWORD
IF '%PASSWORD:~3%'=='' GOTO TOOSHORT


REM ****************************************************************************
REM *** Main routine: iterate all non-encrypted logs                         ***
REM ****************************************************************************

ECHO.
ECHO Encrypting all non-encrypted logs in current directory and subdirectories...
:: We use the following trick instead of using the global variable %CD% because otherwise special characters in the pathname will not be displayed properly (eg: ^).
(ECHO.|SET/P=Current directory: )
CD
ECHO Password used: %PASSWORD%

ECHO.
ECHO Encrypting:
CALL :ENCRYPTALL .
IF EXIST ~EDAF.TMP GOTO NOTCOMPLETE
IF EXIST ~EDAR.TMP GOTO NOTCOMPLETE
GOTO SUCCESS

:ENCRYPTALL
  :: Reason for using this recursive routine and not "FOR /R" or other methods, is so
  :: we can display relative paths, while preserving special characters like ^, (, ), =, etc.
  FOR %%I IN ("%*\*.HTML" "%*\*.TXT") DO (
    ECHO %%I
    :: Use %%~fI instead of %%I to work around a bug in the log viewer involving short filepath names and the "." path indicator.
    %LOGVIEWER% /encrypt="%%~fI" /pwd="%PASSWORD%"
    :: Do not use %ERRORLEVEL% as a variable within a FOR loop.
    IF ERRORLEVEL 1 IF NOT ERRORLEVEL 2 (
      %CMDATTRIB% "%%I" -H -S -R 1>NUL
      DEL "%%I" 1>NUL 2>&1
      IF NOT EXIST "%%I" (
        ECHO %%I >> ~EDAS.TMP
      ) ELSE (
        ECHO %%I >> ~EDAR.TMP
      )
    ) ELSE (
      IF ERRORLEVEL 0 IF NOT ERRORLEVEL 1  ECHO ^(ERR_0^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 1 IF NOT ERRORLEVEL 2  ECHO ^(ERR_1^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 2 IF NOT ERRORLEVEL 3  ECHO ^(ERR_2^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 3 IF NOT ERRORLEVEL 4  ECHO ^(ERR_3^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 4 IF NOT ERRORLEVEL 5  ECHO ^(ERR_4^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 5 IF NOT ERRORLEVEL 6  ECHO ^(ERR_5^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 6 IF NOT ERRORLEVEL 7  ECHO ^(ERR_6^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 7 IF NOT ERRORLEVEL 8  ECHO ^(ERR_7^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 8 IF NOT ERRORLEVEL 9  ECHO ^(ERR_8^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 9 IF NOT ERRORLEVEL 10 ECHO ^(ERR_9^) %%I  >> ~EDAF.TMP
      IF ERRORLEVEL 10                     ECHO ^(ERR_10^) %%I >> ~EDAF.TMP
    )
  )
  FOR /D %%I IN ("%*\*.*") DO CALL %0 %%I
GOTO :EOF


REM ****************************************************************************
REM *** Final messages and error messages                                    ***
REM ****************************************************************************

:NOEXTENSIONS
  ECHO.
  ECHO ERROR: Command extensions can not be enabled.
  ECHO        (possible cause: wrong DOS/Windows version)
GOTO END

:NOCMDPROGS
  ECHO.
  ECHO ERROR: Some needed command tools are not available.
  ECHO        (FIND.EXE, REG.EXE, ATTRIB.EXE)
GOTO END

:NOSAVING
  ECHO.
  ECHO ERROR: Can't create temporary files.
  ECHO        (possible causes: no administration rights, disk write protected, ...)
GOTO END

:NOPLUS
  ECHO.
  ECHO ERROR: Can't find the log viewer program.
  ECHO        (possible cause: Messenger Plus! is not installed properly)
GOTO END

:NOLOGS
  ECHO.
  ECHO There are no non-encrypted logs found in:
  CD
GOTO END

:NOINPUT
  ECHO.
  ECHO Invalid or no input.
GOTO END

:NOPASSWORD
  ECHO.
  ECHO No password given.
GOTO END

:TOOSHORT
  ECHO.
  ECHO Password must be at least 4 characters.
GOTO END

:SUCCESS
  ECHO.
  ECHO All non-encrypted logs are successfully encrypted.
GOTO END

:NOTCOMPLETE
  IF EXIST ~EDAS.TMP (
    ECHO.
    ECHO Encryption and cleanup was successful for:
    TYPE ~EDAS.TMP
  )
  IF EXIST ~EDAR.TMP (
    ECHO.
    ECHO After successful encryption, cleanup has failed for:
    ECHO -^> ^(possible cause: no administration rights^)
    TYPE ~EDAR.TMP
  )
  IF EXIST ~EDAF.TMP (
    ECHO.
    ECHO Encryption has failed for:
    TYPE ~EDAF.TMP
  )
GOTO END

:END
  DEL ~EDAF.TMP 1>NUL 2>&1
  DEL ~EDAR.TMP 1>NUL 2>&1
  DEL ~EDAS.TMP 1>NUL 2>&1
  >&3 ECHO.
  >&3 ECHO Press any key to end...
  >&3 PAUSE 1>NUL
  >&3 ECHO.
  :: don't use "EXIT /B" if command extensions are disabled
